@@ -77,6 +77,11 @@ void bcachefs_usage(void)
 	     "  subvolume snapshot       Create a snapshot\n"
 	     "\n"
 	     "Commands for managing filesystem data:\n"
+	     "  reconcile status         Show status of background data processing\n"
+	     "  reconcile wait           Wait for background data processing (of a specified type) to complete\n"
+	     "  scrub                    Verify checksums and correct errors, if possible\n"
+	     "\n"
+	     "Commands for managing filesystem data (obsolete):\n"
 	     "  data rereplicate         Rereplicate degraded data\n"
 	     "  data scrub               Verify checksums and correct errors, if possible\n"
 	     "  data job                 Kick off low level data jobs\n"
