@@ -3,14 +3,18 @@
 #include <getopt.h>
 #include <sys/uio.h>
 #include <unistd.h>
+
 #include "cmds.h"
-#include "libbcachefs/error.h"
 #include "libbcachefs.h"
-#include "libbcachefs/recovery_passes.h"
-#include "libbcachefs/super.h"
-#include "libbcachefs/super-io.h"
 #include "tools-util.h"
 
+#include "fs/check.h"
+
+#include "init/error.h"
+#include "init/fs.h"
+#include "init/passes.h"
+#include "sb/io.h"
+
 static void setnonblocking(int fd)
 {
 	int flags = fcntl(fd, F_GETFL);
@@ -152,7 +156,7 @@ static bool should_use_kernel_fsck(darray_const_str devs)
 		printbuf_exit(&buf);
 	}
 
-	bch2_fs_stop(c);
+	bch2_fs_exit(c);
 
 	return ret;
 }
@@ -202,15 +206,16 @@ static void fsck_usage(void)
 	     "Usage: bcachefs fsck [OPTION]... <devices>\n"
 	     "\n"
 	     "Options:\n"
-	     "  -p                      Automatic repair (no questions)\n"
-	     "  -n                      Don't repair, only check for errors\n"
-	     "  -y                      Assume \"yes\" to all questions\n"
-	     "  -f                      Force checking even if filesystem is marked clean\n"
-	     "  -r, --ratelimit_errors  Don't display more than 10 errors of a given type\n"
-	     "  -k, --kernel            Use the in-kernel fsck implementation\n"
+	     "  -p                           Automatic repair (no questions)\n"
+	     "  -n                           Don't repair, only check for errors\n"
+	     "  -y                           Assume \"yes\" to all questions\n"
+	     "  -f                           Force checking even if filesystem is marked clean\n"
+	     "  -r, --ratelimit_errors       Don't display more than 10 errors of a given type\n"
+	     "  -k, --kernel                 Use the in-kernel fsck implementation\n"
 	     "  -K, --no-kernel\n"
-	     "  -v                      Be verbose\n"
-	     "  -h, --help              Display this help and exit\n"
+	     "  -v                           Be verbose\n"
+	     "  -h, --help                   Display this help and exit\n"
+	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
 }
 
@@ -225,7 +230,7 @@ int cmd_fsck(int argc, char *argv[])
 	};
 	int kernel = -1; /* unset */
 	int opt, ret = 0;
-	struct printbuf opts_str = PRINTBUF;
+	CLASS(printbuf, opts_str)();
 
 	if (getenv("BCACHEFS_KERNEL_ONLY"))
 		kernel = true;
@@ -357,19 +362,18 @@ userland_fsck:
 		if (IS_ERR(c))
 			exit(8);
 
-		if (test_bit(BCH_FS_errors_fixed, &c->flags)) {
-			fprintf(stderr, "%s: errors fixed\n", c->name);
-			ret |= 1;
-		}
-		if (test_bit(BCH_FS_error, &c->flags)) {
-			fprintf(stderr, "%s: still has errors\n", c->name);
-			ret |= 4;
-		}
+		CLASS(printbuf, buf)();
+		ret = bch2_fs_fsck_errcode(c, &buf);
+		if (ret)
+			fputs(buf.buf, stderr);
 
-		bch2_fs_stop(c);
+		int ret2 = bch2_fs_exit(c);
+		if (ret2) {
+			fprintf(stderr, "error shutting down filesystem: %s\n", bch2_err_str(ret2));
+			ret |= 8;
+		}
 	}
 
-	printbuf_exit(&opts_str);
 	return ret;
 }
 
@@ -381,9 +385,10 @@ static void recovery_pass_usage(void)
 	     "Currently only supports unmounted/offline filesystems\n"
 	     "\n"
 	     "Options:\n"
-	     "  -s, --set               Schedule a recovery pass in the superblock\n"
-	     "  -u, --unset             Deschedule a recovery pass\n"
-	     "  -h, --help              Display this help and exit\n"
+	     "  -s, --set                    Schedule a recovery pass in the superblock\n"
+	     "  -u, --unset                  Deschedule a recovery pass\n"
+	     "  -h, --help                   Display this help and exit\n"
+	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
 }
 
@@ -459,7 +464,7 @@ int cmd_recovery_pass(int argc, char *argv[])
 		printf("%s\n", buf.buf);
 	}
 err:
-	bch2_fs_stop(c);
+	bch2_fs_exit(c);
 	return ret;
 }
 
