@@ -21,14 +21,15 @@
 
 #include "cmds.h"
 #include "cmd_strip_alloc.h"
-#include "libbcachefs/errcode.h"
-#include "libbcachefs/opts.h"
-#include "libbcachefs/journal.h"
-#include "libbcachefs/sb-clean.h"
-#include "libbcachefs/super-io.h"
-#include "libbcachefs/util.h"
 
-#include "libbcachefs/darray.h"
+#include "errcode.h"
+#include "opts.h"
+#include "init/fs.h"
+#include "journal/journal.h"
+#include "sb/clean.h"
+#include "sb/io.h"
+#include "util/util.h"
+#include "util/darray.h"
 
 void strip_fs_alloc(struct bch_fs *c)
 {
@@ -51,6 +52,12 @@ void strip_fs_alloc(struct bch_fs *c)
 	swap(u64s, clean->field.u64s);
 	bch2_sb_field_resize(&c->disk_sb, clean, u64s);
 
+	scoped_guard(percpu_write, &c->capacity.mark_lock) {
+		kfree(c->replicas.entries);
+		c->replicas.entries = NULL;
+		c->replicas.nr = 0;
+	}
+
 	bch2_sb_field_resize(&c->disk_sb, replicas_v0, 0);
 	bch2_sb_field_resize(&c->disk_sb, replicas, 0);
 
@@ -76,6 +83,7 @@ static void strip_alloc_usage(void)
 	     "\n"
 	     "Options:\n"
 	     "  -h, --help              Display this help and exit\n"
+	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
 }
 
@@ -118,7 +126,7 @@ reopen:
 			fprintf(stderr, "Error starting filesystem: %s\n", bch2_err_str(ret));
 			goto err_stop;
 		}
-		bch2_fs_stop(c);
+		bch2_fs_exit(c);
 		goto reopen;
 	}
 
@@ -138,6 +146,6 @@ reopen:
 	bch2_write_super(c);
 	mutex_unlock(&c->sb_lock);
 err_stop:
-	bch2_fs_stop(c);
+	bch2_fs_exit(c);
 	return ret;
 }
