@@ -17,11 +17,12 @@
 #include <blkid.h>
 #include <uuid/uuid.h>
 
+#include "bcachefs_ioctl.h"
+#include "util/util.h"
+
 #include "libbcachefs.h"
-#include "libbcachefs/bcachefs_ioctl.h"
 #include "linux/sort.h"
 #include "tools-util.h"
-#include "libbcachefs/util.h"
 #include "src/rust_to_c.h"
 
 void die(const char *fmt, ...)
@@ -647,10 +648,12 @@ char *fd_to_dev_model(int fd)
 		free(sysfs_path);
 		return strdup("(unknown model)");
 got_model:
-		char* model = read_file_str(AT_FDCWD, model_path);
-		free(model_path);
-		free(sysfs_path);
-		return model;
+		{
+			char *model = read_file_str(AT_FDCWD, model_path);
+			free(model_path);
+			free(sysfs_path);
+			return model;
+		}
 	} else {
 		return strdup("(image file)");
 	}
