@@ -16,6 +16,8 @@
 
 #include "init/error.h"
 
+#include "sb/counters.h"
+
 #include "snapshots/snapshot.h"
 
 #include <linux/string_helpers.h>
@@ -314,6 +316,11 @@ __btree_trans_update_by_path(struct btree_trans *trans,
 	EBUG_ON(!path->should_be_locked);
 	EBUG_ON(trans->nr_updates >= trans->nr_paths);
 	EBUG_ON(!bpos_eq(k->k.p, path->pos));
+	EBUG_ON(!path->level &&
+		btree_type_has_snapshots(path->btree_id) &&
+		!bkey_deleted(&k->k) &&
+		test_bit(JOURNAL_replay_done, &c->journal.flags) &&
+		!bch2_snapshot_exists(c, k->k.p.snapshot));
 
 	n = (struct btree_insert_entry) {
 		.flags		= flags,
@@ -374,7 +381,12 @@ __btree_trans_update_by_path(struct btree_trans *trans,
 
 	__btree_path_get(trans, trans->paths + i->path, true);
 
-	trace_update_by_path(trans, path, i, overwrite);
+	event_trace(c, update_by_path, buf, ({
+		prt_printf(&buf, "%s overwrite %u\n", trans->fn, overwrite);
+		bch2_btree_path_to_text_short(&buf, trans, path_idx, path);
+		bch2_bkey_val_to_text(&buf, c, bkey_i_to_s_c(k));
+	}));
+
 	return i;
 }
 
@@ -383,14 +395,11 @@ static noinline int flush_new_cached_update(struct btree_trans *trans,
 					    enum btree_iter_update_trigger_flags flags,
 					    unsigned long ip)
 {
-	btree_path_idx_t path_idx =
-		bch2_path_get(trans, i->btree_id, i->old_k.p, 1, 0,
-			      BTREE_ITER_intent, _THIS_IP_);
-	int ret = bch2_btree_path_traverse(trans, path_idx, 0);
-	if (ret)
-		goto out;
+	CLASS(btree_iter, iter)(trans, i->btree_id, i->old_k.p, BTREE_ITER_intent);
 
-	struct btree_path *btree_path = trans->paths + path_idx;
+	try(bch2_btree_iter_traverse(&iter));
+
+	struct btree_path *btree_path = btree_iter_path(trans, &iter);
 
 	btree_path_set_should_be_locked(trans, btree_path);
 #if 0
@@ -403,7 +412,7 @@ static noinline int flush_new_cached_update(struct btree_trans *trans,
 	struct bkey k;
 	bch2_btree_path_peek_slot_exact(btree_path, &k);
 	if (!bkey_deleted(&k))
-		goto out;
+		return 0;
 #endif
 	i->key_cache_already_flushed = true;
 	i->flags |= BTREE_TRIGGER_norun;
@@ -411,14 +420,18 @@ static noinline int flush_new_cached_update(struct btree_trans *trans,
 	struct bkey old_k		= i->old_k;
 	const struct bch_val *old_v	= i->old_v;
 
-	i = __btree_trans_update_by_path(trans, path_idx, i->k, flags, _THIS_IP_);
+	i = __btree_trans_update_by_path(trans, iter.path, i->k, flags, ip);
 
 	i->old_k		= old_k;
 	i->old_v		= old_v;
 	i->key_cache_flushing	= true;
-out:
-	bch2_path_put(trans, path_idx, true);
-	return ret;
+	return 0;
+}
+
+static inline bool key_cache_needs_flush(struct btree_path *path)
+{
+	struct bkey_cached *ck = (void *) path->l[0].b;
+	return ck->needs_immediate_flush;
 }
 
 static int __must_check
@@ -435,7 +448,9 @@ bch2_trans_update_by_path(struct btree_trans *trans, btree_path_idx_t path_idx,
 	 * the key cache - but the key has to exist in the btree for that to
 	 * work:
 	 */
-	return i->cached && (!i->old_btree_u64s || bkey_deleted(&k->k))
+	return i->cached && (!i->old_btree_u64s ||
+			     bkey_deleted(&k->k) ||
+			     key_cache_needs_flush(trans->paths + path_idx))
 		? flush_new_cached_update(trans, i, flags, ip)
 		: 0;
 }
@@ -465,7 +480,7 @@ static noinline int bch2_trans_update_get_key_cache(struct btree_trans *trans,
 		struct bkey_cached *ck = (void *) trans->paths[iter->key_cache_path].l[0].b;
 
 		if (test_bit(BKEY_CACHED_DIRTY, &ck->flags)) {
-			trace_and_count(trans->c, trans_restart_key_cache_raced, trans, _RET_IP_);
+			event_inc_trace(trans->c, trans_restart_key_cache_raced, buf, prt_str(&buf, trans->fn));
 			return btree_trans_restart(trans, BCH_ERR_transaction_restart_key_cache_raced);
 		}
 
@@ -482,7 +497,6 @@ int __must_check bch2_trans_update_ip(struct btree_trans *trans, struct btree_it
 	kmsan_check_memory(k, bkey_bytes(&k->k));
 
 	btree_path_idx_t path_idx = iter->update_path ?: iter->path;
-	int ret;
 
 	if (iter->flags & BTREE_ITER_is_extents)
 		return bch2_trans_update_extent(trans, iter, k, flags);
@@ -490,7 +504,7 @@ int __must_check bch2_trans_update_ip(struct btree_trans *trans, struct btree_it
 	if (bkey_deleted(&k->k) &&
 	    !(flags & BTREE_UPDATE_key_cache_reclaim) &&
 	    (iter->flags & BTREE_ITER_filter_snapshots)) {
-		ret = need_whiteout_for_snapshot(trans, iter->btree_id, k->k.p);
+		int ret = need_whiteout_for_snapshot(trans, iter->btree_id, k->k.p);
 		if (unlikely(ret < 0))
 			return ret;
 
@@ -505,7 +519,7 @@ int __must_check bch2_trans_update_ip(struct btree_trans *trans, struct btree_it
 	if (!(flags & BTREE_UPDATE_key_cache_reclaim) &&
 	    !path->cached &&
 	    !path->level &&
-	    btree_id_cached(trans->c, path->btree_id)) {
+	    btree_id_cached(path->btree_id)) {
 		try(bch2_trans_update_get_key_cache(trans, iter, path));
 
 		path_idx = iter->key_cache_path;
@@ -591,7 +605,7 @@ int bch2_btree_insert_nonextent(struct btree_trans *trans,
 				BTREE_ITER_not_extents|
 				BTREE_ITER_intent);
 	return  bch2_btree_iter_traverse(&iter) ?:
-		bch2_trans_update(trans, &iter, k, flags);
+		bch2_trans_update_ip(trans, &iter, k, flags, _RET_IP_);
 }
 
 int bch2_btree_insert_trans(struct btree_trans *trans, enum btree_id btree,
@@ -600,7 +614,7 @@ int bch2_btree_insert_trans(struct btree_trans *trans, enum btree_id btree,
 	CLASS(btree_iter, iter)(trans, btree, bkey_start_pos(&k->k),
 				BTREE_ITER_intent|flags);
 	return  bch2_btree_iter_traverse(&iter) ?:
-		bch2_trans_update(trans, &iter, k, flags);
+		bch2_trans_update_ip(trans, &iter, k, flags, _RET_IP_);
 }
 
 /**
