@@ -19,6 +19,8 @@
 #include "journal/seq_blacklist.h"
 #include "journal/write.h"
 
+#include "sb/counters.h"
+
 #include "util/enumerated_ref.h"
 
 static bool __journal_entry_is_open(union journal_res_state state)
@@ -151,30 +153,28 @@ journal_error_check_stuck(struct journal *j, int error, unsigned flags)
 	return stuck;
 }
 
+void bch2_journal_do_writes_locked(struct journal *j)
+{
+	lockdep_assert_held(&j->lock);
+
+	struct bch_fs *c = container_of(j, struct bch_fs, journal);
+	u64 seq = journal_last_unallocated_seq(j);
+	struct journal_buf *w = j->buf + (seq & JOURNAL_BUF_MASK);
+
+	if (seq &&
+	    !w->write_started &&
+	    !journal_state_seq_count(j, j->reservations, seq)) {
+		j->seq_write_started = seq;
+		w->write_started = true;
+		closure_get(&c->cl);
+		closure_call(&w->io, bch2_journal_write, j->wq, NULL);
+	}
+}
+
 void bch2_journal_do_writes(struct journal *j)
 {
-	struct bch_fs *c = container_of(j, struct bch_fs, journal);
-
-	for (u64 seq = journal_last_unwritten_seq(j);
-	     seq <= journal_cur_seq(j);
-	     seq++) {
-		unsigned idx = seq & JOURNAL_BUF_MASK;
-		struct journal_buf *w = j->buf + idx;
-
-		if (w->write_started && !w->write_allocated)
-			break;
-		if (w->write_started)
-			continue;
-
-		if (!journal_state_seq_count(j, j->reservations, seq)) {
-			j->seq_write_started = seq;
-			w->write_started = true;
-			closure_get(&c->cl);
-			closure_call(&w->io, bch2_journal_write, j->wq, NULL);
-		}
-
-		break;
-	}
+	guard(spinlock)(&j->lock);
+	bch2_journal_do_writes_locked(j);
 }
 
 /*
@@ -187,8 +187,8 @@ void bch2_journal_buf_put_final(struct journal *j, u64 seq)
 	lockdep_assert_held(&j->lock);
 
 	if (__bch2_journal_pin_put(j, seq))
-		bch2_journal_reclaim_fast(j);
-	bch2_journal_do_writes(j);
+		bch2_journal_update_last_seq(j);
+	bch2_journal_do_writes_locked(j);
 
 	/*
 	 * for __bch2_next_write_buffer_flush_journal_buf(), when quiescing an
@@ -235,21 +235,20 @@ static void __journal_entry_close(struct journal *j, unsigned closed_val, bool t
 	/* Close out old buffer: */
 	buf->data->u64s		= cpu_to_le32(old.cur_entry_offset);
 
-	struct journal_entry_pin_list *pin_list =
-		journal_seq_pin(j, journal_cur_seq(j));
-	pin_list->bytes = roundup_pow_of_two(vstruct_bytes(buf->data));
-	j->dirty_entry_bytes += pin_list->bytes;
+	size_t bytes = roundup_pow_of_two(vstruct_bytes(buf->data));
 
-	if (trace_journal_entry_close_enabled() && trace) {
-		CLASS(printbuf, err)();
-		guard(printbuf_atomic)(&err);
+	journal_seq_pin(j, journal_cur_seq(j))->bytes = bytes;
+	j->dirty_entry_bytes += bytes;
 
-		prt_str(&err, "entry size: ");
-		prt_human_readable_u64(&err, vstruct_bytes(buf->data));
-		prt_newline(&err);
-		bch2_prt_task_backtrace(&err, current, 1, GFP_NOWAIT);
-		trace_journal_entry_close(c, err.buf);
-	}
+	if (trace)
+		event_trace(c, journal_entry_close, msg, ({
+			guard(printbuf_atomic)(&msg);
+
+			prt_str(&msg, "entry size: ");
+			prt_human_readable_u64(&msg, vstruct_bytes(buf->data));
+			prt_newline(&msg);
+			bch2_prt_task_backtrace(&msg, current, 1, GFP_NOWAIT);
+		}));
 
 	sectors = vstruct_blocks_plus(buf->data, c->block_bits,
 				      buf->u64s_reserved) << c->block_bits;
@@ -280,7 +279,7 @@ static void __journal_entry_close(struct journal *j, unsigned closed_val, bool t
 	 * contain either what the old pin protected or what the new pin
 	 * protects.
 	 *
-	 * After the old pin is dropped journal_last_seq() won't include the old
+	 * After the old pin is dropped j->last_seq won't include the old
 	 * pin, so we can only write the updated last_seq on the entry that
 	 * contains whatever the new pin protects.
 	 *
@@ -291,7 +290,7 @@ static void __journal_entry_close(struct journal *j, unsigned closed_val, bool t
 	 * Hence, we want update/set last_seq on the current journal entry right
 	 * before we open a new one:
 	 */
-	buf->last_seq		= journal_last_seq(j);
+	buf->last_seq		= j->last_seq;
 	buf->data->last_seq	= cpu_to_le64(buf->last_seq);
 	BUG_ON(buf->last_seq > le64_to_cpu(buf->data->seq));
 
@@ -318,7 +317,7 @@ void bch2_journal_halt(struct journal *j)
 	bch2_journal_halt_locked(j);
 }
 
-static bool journal_entry_want_write(struct journal *j)
+static bool journal_entry_close_locked(struct journal *j)
 {
 	bool ret = !journal_entry_is_open(j) ||
 		journal_cur_seq(j) == journal_last_unwritten_seq(j);
@@ -341,7 +340,7 @@ static bool journal_entry_want_write(struct journal *j)
 bool bch2_journal_entry_close(struct journal *j)
 {
 	guard(spinlock)(&j->lock);
-	return journal_entry_want_write(j);
+	return journal_entry_close_locked(j);
 }
 
 /*
@@ -358,7 +357,6 @@ static int journal_entry_open(struct journal *j)
 
 	lockdep_assert_held(&j->lock);
 	BUG_ON(journal_entry_is_open(j));
-	BUG_ON(c->sb.clean);
 
 	if (j->blocked)
 		return bch_err_throw(c, journal_blocked);
@@ -416,7 +414,7 @@ static int journal_entry_open(struct journal *j)
 
 	/*
 	 * The fifo_push() needs to happen at the same time as j->seq is
-	 * incremented for journal_last_seq() to be calculated correctly
+	 * incremented for j->last_seq to be calculated correctly
 	 */
 	atomic64_inc(&j->seq);
 	journal_pin_list_init(fifo_push_ref(&j->pin), 1);
@@ -435,14 +433,16 @@ static int journal_entry_open(struct journal *j)
 
 	bkey_extent_init(&buf->key);
 	buf->noflush		= false;
-	buf->must_flush		= false;
+	/* if filesystem is clean, the first journal write must be a flush */
+	buf->must_flush		= c->sb.clean;
 	buf->separate_flush	= false;
 	buf->flush_time		= 0;
+	buf->failed.nr		= 0;
 	buf->need_flush_to_write_buffer = true;
 	buf->write_started	= false;
 	buf->write_allocated	= false;
 	buf->write_done		= false;
-	buf->had_error		= false;
+	buf->empty		= false;
 
 	memset(buf->data, 0, sizeof(*buf->data));
 	buf->data->seq	= cpu_to_le64(journal_cur_seq(j));
@@ -489,10 +489,11 @@ static int journal_entry_open(struct journal *j)
 
 static bool journal_quiesced(struct journal *j)
 {
+	guard(spinlock)(&j->lock);
 	bool ret = atomic64_read(&j->seq) == j->seq_ondisk;
 
 	if (!ret)
-		bch2_journal_entry_close(j);
+		journal_entry_close_locked(j);
 	return ret;
 }
 
@@ -611,37 +612,19 @@ out:
 	if (journal_error_check_stuck(j, ret, flags))
 		ret = bch_err_throw(c, journal_stuck);
 
-	if (ret == -BCH_ERR_journal_max_in_flight &&
-	    track_event_change(&c->times[BCH_TIME_blocked_journal_max_in_flight], true) &&
-	    trace_journal_entry_full_enabled()) {
-		CLASS(printbuf, buf)();
+	if ((ret == -BCH_ERR_journal_max_in_flight &&
+	     track_event_change(&c->times[BCH_TIME_blocked_journal_max_in_flight], true)) ||
+	    (ret == -BCH_ERR_journal_max_open &&
+	     track_event_change(&c->times[BCH_TIME_blocked_journal_max_open], true)))
+		event_inc_trace(c, journal_entry_full, buf, ({
+			prt_printf(&buf, "%s\n", bch2_err_str(ret));
+			bch2_printbuf_make_room(&buf, 4096);
 
-		bch2_printbuf_make_room(&buf, 4096);
-
-		scoped_guard(spinlock, &j->lock) {
-			prt_printf(&buf, "seq %llu\n", journal_cur_seq(j));
-			bch2_journal_bufs_to_text(&buf, j);
-		}
-
-		trace_journal_entry_full(c, buf.buf);
-		count_event(c, journal_entry_full);
-	}
-
-	if (ret == -BCH_ERR_journal_max_open &&
-	    track_event_change(&c->times[BCH_TIME_blocked_journal_max_open], true) &&
-	    trace_journal_entry_full_enabled()) {
-		CLASS(printbuf, buf)();
-
-		bch2_printbuf_make_room(&buf, 4096);
-
-		scoped_guard(spinlock, &j->lock) {
-			prt_printf(&buf, "seq %llu\n", journal_cur_seq(j));
-			bch2_journal_bufs_to_text(&buf, j);
-		}
-
-		trace_journal_entry_full(c, buf.buf);
-		count_event(c, journal_entry_full);
-	}
+			scoped_guard(spinlock, &j->lock) {
+				prt_printf(&buf, "seq %llu\n", journal_cur_seq(j));
+				bch2_journal_bufs_to_text(&buf, j);
+			}
+		}));
 
 	/*
 	 * Journal is full - can't rely on reclaim from work item due to
@@ -670,7 +653,7 @@ static unsigned max_dev_latency(struct bch_fs *c)
 	u64 nsecs = 0;
 
 	guard(rcu)();
-	for_each_member_device_rcu(c, ca, &c->rw_devs[BCH_DATA_journal])
+	for_each_member_device_rcu(c, ca, &c->allocator.rw_devs[BCH_DATA_journal])
 		nsecs = max(nsecs, ca->io_latency[WRITE].stats.max_duration);
 
 	return nsecs_to_jiffies(nsecs);
@@ -818,11 +801,14 @@ recheck_need_open:
 
 		seq = res.seq;
 		buf = journal_seq_to_buf(j, seq);
-		buf->must_flush = true;
 
-		if (!buf->flush_time) {
-			buf->flush_time	= local_clock() ?: 1;
-			buf->expires = jiffies;
+		scoped_guard(spinlock, &j->lock) {
+			buf->must_flush = true;
+
+			if (!buf->flush_time) {
+				buf->flush_time	= local_clock() ?: 1;
+				buf->expires = jiffies;
+			}
 		}
 
 		if (parent && !closure_wait(&buf->wait, parent))
@@ -851,7 +837,7 @@ recheck_need_open:
 		BUG();
 want_write:
 	if (seq == journal_cur_seq(j))
-		journal_entry_want_write(j);
+		journal_entry_close_locked(j);
 out:
 	spin_unlock(&j->lock);
 	return ret;
@@ -934,11 +920,14 @@ int __bch2_journal_meta(struct journal *j)
 	try(bch2_journal_res_get(j, &res, jset_u64s(0), 0, NULL));
 
 	struct journal_buf *buf = j->buf + (res.seq & JOURNAL_BUF_MASK);
-	buf->must_flush = true;
 
-	if (!buf->flush_time) {
-		buf->flush_time	= local_clock() ?: 1;
-		buf->expires = jiffies;
+	scoped_guard(spinlock, &j->lock) {
+		buf->must_flush = true;
+
+		if (!buf->flush_time) {
+			buf->flush_time	= local_clock() ?: 1;
+			buf->expires = jiffies;
+		}
 	}
 
 	bch2_journal_res_put(j, &res);
@@ -1092,9 +1081,10 @@ void __bch2_journal_debug_to_text(struct printbuf *out, struct journal *j)
 	prt_printf(out, "dirty journal entries:\t%llu/%llu\n",	fifo_used(&j->pin), j->pin.size);
 	prt_printf(out, "seq:\t%llu\n",				journal_cur_seq(j));
 	prt_printf(out, "seq_ondisk:\t%llu\n",			j->seq_ondisk);
-	prt_printf(out, "last_seq:\t%llu\n",			journal_last_seq(j));
+	prt_printf(out, "last_seq:\t%llu\n",			j->last_seq);
 	prt_printf(out, "last_seq_ondisk:\t%llu\n",		j->last_seq_ondisk);
 	prt_printf(out, "flushed_seq_ondisk:\t%llu\n",		j->flushed_seq_ondisk);
+	prt_printf(out, "last_empty_seq:\t%llu\n",		j->last_empty_seq);
 	prt_printf(out, "watermark:\t%s\n",			bch2_watermarks[j->watermark]);
 	prt_printf(out, "each entry reserved:\t%u\n",		j->entry_u64s_reserved);
 	prt_printf(out, "nr flush writes:\t%llu\n",		j->nr_flush_writes);
@@ -1147,7 +1137,7 @@ void __bch2_journal_debug_to_text(struct printbuf *out, struct journal *j)
 		       j->space[journal_space_total].total);
 	}
 
-	for_each_member_device_rcu(c, ca, &c->rw_devs[BCH_DATA_journal]) {
+	for_each_member_device_rcu(c, ca, &c->allocator.rw_devs[BCH_DATA_journal]) {
 		struct journal_device *ja = &ca->journal;
 		if (!ja->nr)
 			continue;
