@@ -13,7 +13,7 @@
 
 static int bch2_snapshot_table_make_room(struct bch_fs *c, u32 id)
 {
-	guard(mutex)(&c->snapshot_table_lock);
+	guard(mutex)(&c->snapshots.table_lock);
 	return bch2_snapshot_t_mut(c, id)
 		? 0
 		: bch_err_throw(c, ENOMEM_mark_snapshot);
@@ -34,32 +34,29 @@ static int bch2_snapshot_tree_create(struct btree_trans *trans,
 	return 0;
 }
 
-u32 bch2_snapshot_oldest_subvol(struct bch_fs *c, u32 snapshot_root,
-				snapshot_id_list *skip)
+static u32 bch2_snapshot_oldest_subvol(struct bch_fs *c, u32 snapshot_root,
+				       snapshot_id_list *skip)
 {
 	guard(rcu)();
-	struct snapshot_table *t = rcu_dereference(c->snapshots);
-	u32 id, subvol = 0, s;
-retry:
-	id = snapshot_root;
-	while (id && __bch2_snapshot_exists(t, id)) {
-		if (!(skip && snapshot_list_has_id(skip, id))) {
-			s = __snapshot_t(t, id)->subvol;
+	struct snapshot_table *t = rcu_dereference(c->snapshots.table);
 
+	while (true) {
+		u32 subvol = 0;
+
+		__for_each_snapshot_child(t, snapshot_root, NULL, id)  {
+			if (skip && snapshot_list_has_id(skip, id))
+				continue;
+
+			u32 s = __snapshot_t(t, id)->subvol;
 			if (s && (!subvol || s < subvol))
 				subvol = s;
 		}
-		id = bch2_snapshot_tree_next(t, id);
-		if (id == snapshot_root)
-			break;
-	}
 
-	if (!subvol && skip) {
+		if (subvol || !skip)
+			return subvol;
+
 		skip = NULL;
-		goto retry;
 	}
-
-	return subvol;
 }
 
 static int bch2_snapshot_tree_master_subvol(struct btree_trans *trans,
@@ -116,7 +113,7 @@ static int check_snapshot_tree(struct btree_trans *trans,
 
 	struct bch_snapshot s;
 	if (!ret)
-		bkey_val_copy(&s, snapshot_k);
+		bkey_val_copy_pad(&s, snapshot_k);
 
 	if (fsck_err_on(ret ||
 			root_id != bch2_snapshot_root(c, root_id) ||
@@ -550,8 +547,8 @@ int bch2_reconstruct_snapshots(struct bch_fs *c)
 	struct snapshot_tree_reconstruct r __cleanup(snapshot_tree_reconstruct_exit) = {};
 	int ret = 0;
 
-	struct progress_indicator_state progress;
-	bch2_progress_init(&progress, c, btree_has_snapshots_mask);
+	struct progress_indicator progress;
+	bch2_progress_init(&progress, __func__, c, btree_has_snapshots_mask, 0);
 
 	for (unsigned btree = 0; btree < BTREE_ID_NR; btree++) {
 		if (btree_type_has_snapshots(btree)) {
@@ -559,7 +556,7 @@ int bch2_reconstruct_snapshots(struct bch_fs *c)
 
 			try(for_each_btree_key(trans, iter, btree, POS_MIN,
 					BTREE_ITER_all_snapshots|BTREE_ITER_prefetch, k, ({
-				progress_update_iter(trans, &progress, &iter);
+				bch2_progress_update_iter(trans, &progress, &iter) ?:
 				get_snapshot_trees(c, &r, k.k->p);
 			})));
 
